@@ -6,7 +6,7 @@
 
 from PyQt4.QtGui import *
 from PyQt4.QtCore import *
-import os, sys, cPickle, locale, types, shutil, time
+import os, sys, cPickle, locale, types, shutil, time, re
 from anki.utils import genID
 
 # compatability
@@ -41,55 +41,67 @@ class Config(dict):
 
     def defaults(self):
         fields = {
-            'iconSize': 32,
-            'syncOnLoad': True,
-            'syncOnClose': True,
-            'syncInMsgBox': False,
-            'checkForUpdates': True,
-            'interfaceLang': "",
-            'syncUsername': "",
-            'syncPassword': "",
-            'showFontPreview': False,
-            'showToolbar': True,
-            'recentDeckPaths': [],
-            'saveAfterAnswer': True,
-            'saveAfterAnswerNum': 10,
-            'saveAfterAdding': True,
-            'saveAfterAddingNum': 1,
-            'saveOnClose': True,
-            'mainWindowGeom': None,
-            'mainWindowState': None,
-            'suppressUpdate': False,
-            'suppressEstimates': False,
-            'showLastCardInterval': False,
-            'showLastCardContent': False,
-            'showTrayIcon': False,
-            'showTimer': True,
-            'simpleToolbar': True,
-            'scrollToAnswer': True,
-            'qaDivider': True,
-            'splitQA': True,
-            'sortIndex': 0,
             'addZeroSpace': False,
             'alternativeTheme': False,
-            'showStudyScreen': True,
-            'showStudyOptions': False,
-            'showStudyStats': True,
-            'showCardTimer': True,
-            'standaloneWindows': True,
-            'extraNewCards': 5,
-            'randomizeOnCram': True,
+            'checkForUpdates': True,
             'created': time.time(),
-            'id': genID(),
-            'editorReverseOrder': False,
+            'deckBrowserNameLength': 30,
+            'deckBrowserOrder': 0,
+            'deckBrowserRefreshPeriod': 3600,
+            'deleteMedia': False,
             'editFontFamily': 'Arial',
             'editFontSize': 12,
             'editLineSize': 20,
+            'editorReverseOrder': False,
+            'extraNewCards': 5,
             'factEditorAdvanced': False,
-            'typeAnswerFontSize': 20,
-            'showProgress': True,
-            'recentColours': ["#000000", "#0000ff"],
+            'forceLTR': False,
+            'iconSize': 32,
+            'id': genID(),
+            'interfaceLang': "",
+            'loadLastDeck': False,
+            'mainWindowGeom': None,
+            'mainWindowState': None,
+            'numBackups': 30,
             'preventEditUntilAnswer': False,
+            'proxyHost': '',
+            'proxyPass': '',
+            'proxyPort': 8080,
+            'proxyUser': '',
+            'qaDivider': True,
+            'randomizeOnCram': True,
+            'recentColours': ["#000000", "#0000ff"],
+            'recentDeckPaths': [],
+            'repeatQuestionAudio': True,
+            'saveAfterAdding': True,
+            'saveAfterAddingNum': 1,
+            'saveAfterAnswer': True,
+            'saveAfterAnswerNum': 10,
+            'saveOnClose': True,
+            'scrollToAnswer': True,
+            'showCardTimer': True,
+            'showFontPreview': False,
+            'showLastCardContent': False,
+            'showLastCardInterval': False,
+            'showProgress': True,
+            'showStudyOptions': False,
+            'showStudyScreen': True,
+            'showStudyStats': True,
+            'showTimer': True,
+            'showToolbar': True,
+            'showTrayIcon': False,
+            'simpleToolbar': True,
+            'sortIndex': 0,
+            'splitQA': True,
+            'standaloneWindows': True,
+            'suppressEstimates': False,
+            'suppressUpdate': False,
+            'syncInMsgBox': False,
+            'syncOnClose': True,
+            'syncOnLoad': True,
+            'syncPassword': "",
+            'syncUsername': "",
+            'typeAnswerFontSize': 20,
             }
         for (k,v) in fields.items():
             if not self.has_key(k):
@@ -125,6 +137,13 @@ class Config(dict):
             os.unlink(path)
         os.rename(tmpname, path)
 
+    def fixLang(self, lang):
+        if lang and lang not in ("pt_BR", "zh_CN", "zh_TW"):
+            lang = re.sub("(.*)_.*", "\\1", lang)
+        if not lang:
+            lang = "en"
+        return lang
+
     def load(self):
         base = self.configPath
         db = self.getDbPath()
@@ -132,7 +151,7 @@ class Config(dict):
         try:
             f = open(db)
             self.update(cPickle.load(f))
-        except (IOError, EOFError):
+        except:
             # config file was corrupted previously
             pass
         self.defaults()
@@ -141,16 +160,5 @@ class Config(dict):
             s = self['recentDeckPaths'][n]
             if not isinstance(s, types.UnicodeType):
                 self['recentDeckPaths'][n] = unicode(s, sys.getfilesystemencoding())
-        # fix old locale settings
-        if self["interfaceLang"] == "ja":
-            self["interfaceLang"]="ja_JP"
-        elif self["interfaceLang"] == "fr":
-            self["interfaceLang"]="fr_FR"
-        elif self["interfaceLang"] == "en":
-            self["interfaceLang"]="en_US"
-        elif self["interfaceLang"] == "de":
-            self["interfaceLang"]="de_DE"
-        elif self["interfaceLang"] == "es":
-            self["interfaceLang"]="es_ES"
-        elif not self["interfaceLang"]:
-            self["interfaceLang"]="en_US"
+        # fix locale settings
+        self["interfaceLang"] = self.fixLang(self["interfaceLang"])
