@@ -51,13 +51,12 @@ class DeckProperties(QDialog):
         self.dialog.highPriority.setText(self.d.highPriority)
         self.dialog.medPriority.setText(self.d.medPriority)
         self.dialog.lowPriority.setText(self.d.lowPriority)
-        self.dialog.postponing.setText(self.d.suspended)
         # scheduling
         for type in ("hard", "mid", "easy"):
             v = getattr(self.d, type + "IntervalMin")
-            getattr(self.dialog, type + "Min").setText("%0.3f" % v)
+            getattr(self.dialog, type + "Min").setText(str(v))
             v = getattr(self.d, type + "IntervalMax")
-            getattr(self.dialog, type + "Max").setText("%0.3f" % v)
+            getattr(self.dialog, type + "Max").setText(str(v))
         self.dialog.delay0.setText(unicode(self.d.delay0/60.0))
         self.dialog.delay1.setText(unicode(self.d.delay1/60.0))
         self.dialog.delay2.setText(unicode(self.d.delay2))
@@ -73,6 +72,9 @@ class DeckProperties(QDialog):
         # hour shift
         self.dialog.timeOffset.setText(str(
             (self.d.utcOffset - time.timezone) / 60.0 / 60.0))
+        # leeches
+        self.dialog.suspendLeeches.setChecked(self.d.getBool("suspendLeeches"))
+        self.dialog.leechFails.setValue(self.d.getInt("leechFails"))
 
     def drawSourcesTable(self):
         self.dialog.sourcesTable.clear()
@@ -233,12 +235,23 @@ class DeckProperties(QDialog):
         try:
             v = float(self.dialog.delay0.text()) * 60.0
             self.updateField(self.d, 'delay0', v)
-            v = float(self.dialog.delay1.text()) * 60.0
-            self.updateField(self.d, 'delay1', v)
+            v2 = float(self.dialog.delay1.text()) * 60.0
+            if v2 < v:
+                ui.utils.showInfo(_("Again (Young) must be <= Again (Mature)."),
+                                  parent=self.parent)
+                v2 = v
+            self.updateField(self.d, 'delay1', v2)
             v = float(self.dialog.delay2.text())
-            self.updateField(self.d, 'delay2', v)
+            self.updateField(self.d, 'delay2', min(v, 1))
             v = int(self.dialog.failedCardMax.text())
-            self.updateField(self.d, 'failedCardMax', v)
+            self.updateField(self.d, 'failedCardMax', max(v, 2))
+        except ValueError:
+            pass
+        try:
+            self.d.setVar("suspendLeeches",
+                          not not self.dialog.suspendLeeches.isChecked())
+            self.d.setVar("leechFails",
+                          int(self.dialog.leechFails.value()))
         except ValueError:
             pass
         # hour shift
@@ -260,9 +273,6 @@ class DeckProperties(QDialog):
         self.updateField(self.d,
                          "lowPriority",
                          unicode(self.dialog.lowPriority.text()))
-        self.updateField(self.d,
-                         "suspended",
-                         unicode(self.dialog.postponing.text()))
         prioritiesChanged = was != self.d.modified
         # sources
         d = {}
