@@ -5,7 +5,8 @@
 from PyQt4.QtGui import *
 from PyQt4.QtCore import *
 from PyQt4.QtSvg import *
-import re, os, sys, tempfile, urllib2
+from PyQt4.QtWebKit import QWebPage
+import re, os, sys, tempfile, urllib2, ctypes
 from anki.utils import stripHTML, tidyHTML, canonifyTags
 from anki.sound import playFromText
 from ankiqt.ui.sound import getAudio
@@ -13,11 +14,21 @@ import anki.sound
 from ankiqt import ui
 import ankiqt
 from ankiqt.ui.utils import mungeQA, saveGeom, restoreGeom, getBase
-from anki.hooks import addHook, removeHook, runHook
+from anki.hooks import addHook, removeHook, runHook, runFilter
 from sqlalchemy.exceptions import InvalidRequestError
+from PyQt4 import pyqtconfig
+QtConfig = pyqtconfig.Configuration()
 
 clozeColour = "#0000ff"
 
+if sys.platform.startswith("win32"):
+    ActivateKeyboardLayout = ctypes.windll.user32.ActivateKeyboardLayout
+    ActivateKeyboardLayout.restype = ctypes.c_void_p
+    ActivateKeyboardLayout.argtypes = [ctypes.c_void_p, ctypes.c_uint]
+    GetKeyboardLayout = ctypes.windll.user32.GetKeyboardLayout
+    GetKeyboardLayout.restype = ctypes.c_void_p
+    GetKeyboardLayout.argtypes = [ctypes.c_uint]
+
 class FactEditor(object):
     """An editor for new/existing facts.
 
@@ -221,9 +232,24 @@ class FactEditor(object):
         vbox.addWidget(self.fchoose)
         self.fchoose.setStyle(self.plastiqueStyle)
         self.iconsBox.addLayout(vbox)
-        # pictures
+        # cloze
         spc = QSpacerItem(5,5)
         self.iconsBox.addItem(spc)
+        self.cloze = QPushButton(self.widget)
+        self.clozeSC = QShortcut(QKeySequence(_("F9")), self.widget)
+        self.cloze.connect(self.cloze, SIGNAL("clicked()"),
+                                  self.onCloze)
+        self.cloze.connect(self.clozeSC, SIGNAL("activated()"),
+                                  self.onCloze)
+        self.cloze.setToolTip(_("Cloze (F9)"))
+        self.cloze.setFixedWidth(30)
+        self.cloze.setFixedHeight(26)
+        self.cloze.setText("[...]")
+        self.cloze.setFocusPolicy(Qt.NoFocus)
+        self.cloze.setEnabled(False)
+        self.iconsBox.addWidget(self.cloze)
+        self.cloze.setStyle(self.plastiqueStyle)
+        # pictures
         self.addPicture = QPushButton(self.widget)
         self.addPicture.connect(self.addPicture, SIGNAL("clicked()"), self.onAddPicture)
         self.addPicture.setFocusPolicy(Qt.NoFocus)
@@ -240,7 +266,7 @@ class FactEditor(object):
         self.addSound.setShortcut(_("F4"))
         self.addSound.setEnabled(False)
         self.addSound.setIcon(QIcon(":/icons/text-speak.png"))
-        self.addSound.setToolTip(_("Add audio (F4)"))
+        self.addSound.setToolTip(_("Add audio/video (F4)"))
         self.iconsBox.addWidget(self.addSound)
         self.addSound.setStyle(self.plastiqueStyle)
         # sounds
@@ -253,21 +279,9 @@ class FactEditor(object):
         self.recSound.setToolTip(_("Record audio (F5)"))
         self.iconsBox.addWidget(self.recSound)
         self.recSound.setStyle(self.plastiqueStyle)
-        # more
-        self.more = QPushButton(self.widget)
-        self.more.connect(self.more, SIGNAL("clicked()"),
-                                  self.onMore)
-        self.more.setToolTip(_("Show advanced options"))
-        self.more.setText(">>")
-        self.more.setFocusPolicy(Qt.NoFocus)
-        self.more.setEnabled(False)
-        self.more.setFixedWidth(30)
-        self.more.setFixedHeight(26)
-        self.iconsBox.addWidget(self.more)
-        self.more.setStyle(self.plastiqueStyle)
         # preview
         spc = QSpacerItem(5,5)
-        self.iconsBox2.addItem(spc)
+        self.iconsBox.addItem(spc)
         self.preview = QPushButton(self.widget)
         self.previewSC = QShortcut(QKeySequence(_("F2")), self.widget)
         self.preview.connect(self.preview, SIGNAL("clicked()"),
@@ -277,25 +291,19 @@ class FactEditor(object):
         self.preview.setToolTip(_("Preview (F2)"))
         self.preview.setIcon(QIcon(":/icons/document-preview.png"))
         self.preview.setFocusPolicy(Qt.NoFocus)
-        self.preview.setEnabled(False)
-        self.iconsBox2.addWidget(self.preview)
+        self.iconsBox.addWidget(self.preview)
         self.preview.setStyle(self.plastiqueStyle)
-        # cloze
-        self.cloze = QPushButton(self.widget)
-        self.clozeSC = QShortcut(QKeySequence(_("F9")), self.widget)
-        self.cloze.connect(self.cloze, SIGNAL("clicked()"),
-                                  self.onCloze)
-        self.cloze.connect(self.clozeSC, SIGNAL("activated()"),
-                                  self.onCloze)
-        self.cloze.setToolTip(_("Cloze (F9)"))
-        #self.cloze.setIcon(QIcon(":/icons/document-cloze.png"))
-        self.cloze.setFixedWidth(30)
-        self.cloze.setFixedHeight(26)
-        self.cloze.setText("[...]")
-        self.cloze.setFocusPolicy(Qt.NoFocus)
-        self.cloze.setEnabled(False)
-        self.iconsBox2.addWidget(self.cloze)
-        self.cloze.setStyle(self.plastiqueStyle)
+        # more
+        self.more = QPushButton(self.widget)
+        self.more.connect(self.more, SIGNAL("clicked()"),
+                                  self.onMore)
+        self.more.setToolTip(_("Show advanced options"))
+        self.more.setText(">>")
+        self.more.setFocusPolicy(Qt.NoFocus)
+        self.more.setFixedWidth(30)
+        self.more.setFixedHeight(26)
+        self.iconsBox.addWidget(self.more)
+        self.more.setStyle(self.plastiqueStyle)
         # latex
         spc = QSpacerItem(5,5)
         self.iconsBox2.addItem(spc)
@@ -387,6 +395,10 @@ class FactEditor(object):
             w.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Expanding)
             w.setHorizontalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
             w.setVerticalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
+            if field.fieldModel.features:
+                w.setLayoutDirection(Qt.RightToLeft)
+            else:
+                w.setLayoutDirection(Qt.LeftToRight)
             runHook("makeField", w, field)
             self.fieldsGrid.addWidget(w, n, 1)
             self.fields[field.name] = (field, w)
@@ -408,12 +420,16 @@ class FactEditor(object):
         self.loadFields(check)
         self.parent.setUpdatesEnabled(True)
         self.fieldsScroll.setWidget(self.fieldsFrame)
-        extra = 0
         if sys.platform.startswith("darwin"):
             extra = 5
         elif sys.platform.startswith("win32"):
             extra = 3
-        self.tagsLabel.setFixedWidth(max(*[l.width() for l in self.labels])
+        else:
+            extra = 2
+        tagsw = self.tagsLabel.sizeHint().width()
+        self.tagsLabel.setFixedWidth(max(tagsw,
+                                         max(*([
+            l.width() for l in self.labels] + [0])))
                                      + extra)
         self.parent.setTabOrder(last, self.tags)
 
@@ -512,7 +528,7 @@ class FactEditor(object):
         self.saveFields()
         self.checkValid()
         if self.onChange:
-            self.onChange()
+            self.onChange('field')
         self.changeTimer = None
 
     def saveFieldsNow(self):
@@ -524,7 +540,7 @@ class FactEditor(object):
             self.changeTimer.stop()
             self.changeTimer = None
             if self.onChange:
-                self.onChange()
+                self.onChange('field')
         # save fields and run features
         w = self.focusedEdit()
         if w:
@@ -566,31 +582,37 @@ class FactEditor(object):
             return
         old = self.fact.tags
         self.fact.tags = canonifyTags(unicode(self.tags.text()))
-        if self.onChange:
-            self.onChange()
         if old != self.fact.tags:
             self.deck.s.flush()
             self.deck.updateFactTags([self.fact.id])
             self.deck.updatePriorities([c.id for c in self.fact.cards])
             self.fact.setModified(textChanged=True)
             self.deck.flushMod()
+        if self.onChange:
+            self.onChange('tag')
 
     def focusField(self, fieldName):
         self.fields[fieldName][1].setFocus()
 
     def formatChanged(self, fmt):
         w = self.focusedEdit()
-        if not w or w.textCursor().hasSelection():
+        if not w:
             return
         else:
+            l = self.bold, self.italic, self.underline
+            for b in l:
+                b.blockSignals(True)
             self.bold.setChecked(w.fontWeight() == QFont.Bold)
             self.italic.setChecked(w.fontItalic())
             self.underline.setChecked(w.fontUnderline())
+            for b in l:
+                b.blockSignals(False)
 
     def resetFormatButtons(self):
-        self.bold.setChecked(False)
-        self.italic.setChecked(False)
-        self.underline.setChecked(False)
+        for b in self.bold, self.italic, self.underline:
+            b.blockSignals(True)
+            b.setChecked(False)
+            b.blockSignals(False)
 
     def enableButtons(self, val=True):
         self.bold.setEnabled(val)
@@ -605,11 +627,9 @@ class FactEditor(object):
         self.latex.setEnabled(val)
         self.latexEqn.setEnabled(val)
         self.latexMathEnv.setEnabled(val)
-        self.preview.setEnabled(val)
         self.cloze.setEnabled(val)
         self.htmlEdit.setEnabled(val)
         self.recSound.setEnabled(val)
-        self.more.setEnabled(val)
 
     def disableButtons(self):
         self.enableButtons(False)
@@ -683,13 +703,21 @@ class FactEditor(object):
             runHook("colourChanged")
             self.setForeground(w)
 
+    def _needExtraWord(self):
+        ver = QtConfig.qt_version >> 8
+        if ver == 0x404:
+            # qt4.4 behaviour is wrong
+            return False
+        return True
+
     def insertLatex(self):
         w = self.focusedEdit()
         if w:
+            selected = w.textCursor().selectedText()
             self.deck.mediaDir(create=True)
-            w.insertHtml("[latex][/latex]")
+            w.insertHtml("[latex]%s[/latex]" % selected)
             w.moveCursor(QTextCursor.PreviousWord)
-            if sys.platform.startswith("win32"):
+            if self._needExtraWord():
                 w.moveCursor(QTextCursor.PreviousWord)
             w.moveCursor(QTextCursor.PreviousCharacter)
             w.moveCursor(QTextCursor.PreviousCharacter)
@@ -697,10 +725,11 @@ class FactEditor(object):
     def insertLatexEqn(self):
         w = self.focusedEdit()
         if w:
+            selected = w.textCursor().selectedText()
             self.deck.mediaDir(create=True)
-            w.insertHtml("[$][/$]")
+            w.insertHtml("[$]%s[/$]" % selected)
             w.moveCursor(QTextCursor.PreviousWord)
-            if sys.platform.startswith("win32"):
+            if self._needExtraWord():
                 w.moveCursor(QTextCursor.PreviousWord)
             w.moveCursor(QTextCursor.PreviousCharacter)
             w.moveCursor(QTextCursor.PreviousCharacter)
@@ -708,20 +737,19 @@ class FactEditor(object):
     def insertLatexMathEnv(self):
         w = self.focusedEdit()
         if w:
+            selected = w.textCursor().selectedText()
             self.deck.mediaDir(create=True)
-            w.insertHtml("[$$][/$$]")
+            w.insertHtml("[$$]%s[/$$]" % selected)
             w.moveCursor(QTextCursor.PreviousWord)
-            if sys.platform.startswith("win32"):
+            if self._needExtraWord():
                 w.moveCursor(QTextCursor.PreviousWord)
             w.moveCursor(QTextCursor.PreviousCharacter)
             w.moveCursor(QTextCursor.PreviousCharacter)
 
     def onMore(self, toggle=None):
         if toggle is None:
-            toggle = not self.preview.isVisible()
+            toggle = not self.latex.isVisible()
             ankiqt.mw.config['factEditorAdvanced'] = toggle
-        self.preview.setShown(toggle)
-        self.cloze.setShown(toggle)
         self.latex.setShown(toggle)
         self.latexEqn.setShown(toggle)
         self.latexMathEnv.setShown(toggle)
@@ -734,8 +762,8 @@ class FactEditor(object):
         src = self.focusedEdit()
         if not src:
             return
-        re1 = "\[.+?(:(.+?))?\]"
-        re2 = "\[(.+?)(:.+?)?\]"
+        re1 = "\[(?:<.+?>)?.+?(:(.+?))?\](?:</.+?>)?"
+        re2 = "\[(?:<.+?>)?(.+?)(:.+?)?\](?:</.+?>)?"
         # add brackets because selected?
         cursor = src.textCursor()
         oldSrc = None
@@ -776,13 +804,17 @@ class FactEditor(object):
                 self.saveFields()
                 return
             else:
-                ui.utils.showInfo(_("Next field must be blank."),
-                                  parent=self.parent)
+                ui.utils.showInfo(
+                    _("Next field must be blank."),
+                    help="ClozeDeletion",
+                    parent=self.parent)
                 return
         # check if there's anything to change
         if not re.search("\[.+?\]", unicode(src.toPlainText())):
-            QDesktopServices.openUrl(QUrl(ankiqt.appWiki +
-                                          "ClozeDeletion"))
+            ui.utils.showInfo(
+                _("You didn't specify anything to occlude."),
+                help="ClozeDeletion",
+                parent=self.parent)
             return
         # create
         s = unicode(src.toHtml())
@@ -830,7 +862,8 @@ class FactEditor(object):
     def onAddPicture(self):
         # get this before we open the dialog
         w = self.focusedEdit()
-        key = _("Images (*.jpg *.png *.gif *.tiff *.svg *.tif *.jpeg)")
+        key = (_("Images") +
+               " (*.jpg *.png *.gif *.tiff *.svg *.tif *.jpeg)")
         file = ui.utils.getFile(self.parent, _("Add an image"), "picture", key)
         if not file:
             return
@@ -854,12 +887,15 @@ class FactEditor(object):
         else:
             w = self.focusedEdit()
         path = self.deck.addMedia(file)
+        self.maybeDelete(path, file)
         w.insertHtml('<img src="%s">' % path)
 
     def onAddSound(self):
         # get this before we open the dialog
         w = self.focusedEdit()
-        key = _("Sounds (*.mp3 *.ogg *.wav)")
+        key = (_("Sounds/Videos") +
+               " (*.mp3 *.ogg *.wav *.avi *.ogv *.mpg *.mpeg *.mov *.mp4 " +
+               "*.mkv *.ogx *.ogv *.oga)")
         file = ui.utils.getFile(self.parent, _("Add audio"), "audio", key)
         if not file:
             return
@@ -872,9 +908,20 @@ class FactEditor(object):
         else:
             w = self.focusedEdit()
         path = self.deck.addMedia(file)
+        self.maybeDelete(path, file)
         anki.sound.play(path)
         w.insertHtml('[sound:%s]' % path)
 
+    def maybeDelete(self, new, old):
+        if not ankiqt.mw.config['deleteMedia']:
+            return
+        if new == os.path.basename(old):
+            return
+        try:
+            os.unlink(old)
+        except:
+            pass
+
     def onRecSound(self):
         self.initMedia()
         w = self.focusedEdit()
@@ -895,6 +942,8 @@ class FactEdit(QTextEdit):
     def __init__(self, parent, *args):
         QTextEdit.__init__(self, *args)
         self.parent = parent
+        if sys.platform.startswith("win32"):
+            self._ownLayout = None
 
     def canInsertFromMimeData(self, source):
         return (source.hasUrls() or
@@ -917,6 +966,9 @@ class FactEdit(QTextEdit):
                 if "\n" in txt:
                     txt = txt.split("\n")[0]
                     hadN = True
+                if "\r" in txt:
+                    txt = txt.split("\r")[0]
+                    hadN = True
                 if not source.hasImage() or hadN:
                     # firefox on linux just gives us a url
                     ext = txt.split(".")[-1].lower()
@@ -994,48 +1046,11 @@ class FactEdit(QTextEdit):
         self.parent.lastFocusedEdit = self
         self.parent.resetFormatButtons()
         self.parent.disableButtons()
+        if sys.platform.startswith("win32"):
+            self._ownLayout = GetKeyboardLayout(0)
+            ActivateKeyboardLayout(self._programLayout, 0)
         self.emit(SIGNAL("lostFocus"))
 
-    # this shouldn't be necessary if/when we move away from kakasi
-    def mouseDoubleClickEvent(self, evt):
-        t = self.parent.fact.model.tags.lower()
-        if (not "japanese" in t and
-            not "mandarin" in t and
-            not "cantonese" in t):
-            return QTextEdit.mouseDoubleClickEvent(self,evt)
-        r = QRegExp("\\{(.*[|,].*)\\}")
-        r.setMinimal(True)
-
-        mouseposition = self.textCursor().position()
-
-        blockoffset = 0
-        result = r.indexIn(self.toPlainText(), 0)
-
-        found = ""
-
-        while result != -1:
-            if mouseposition > result and mouseposition < result + r.matchedLength():
-                mouseposition -= result + 1
-                frompos = 0
-                topos = 0
-
-                string = r.cap(1)
-                offset = 0
-                bits = re.split("[|,]", unicode(string))
-                for index in range(0, len(bits)):
-                    offset += len(bits[index]) + 1
-                    if mouseposition < offset:
-                        found = bits[index]
-                        break
-                break
-
-            blockoffset= result + r.matchedLength()
-            result = r.indexIn(self.toPlainText(), blockoffset)
-
-        if found == "":
-            return QTextEdit.mouseDoubleClickEvent(self,evt)
-        self.setPlainText(self.toPlainText().replace(result, r.matchedLength(), found))
-
     def focusInEvent(self, evt):
         if (self.parent.lastFocusedEdit and
             self.parent.lastFocusedEdit is not self):
@@ -1051,6 +1066,11 @@ class FactEdit(QTextEdit):
         QTextEdit.focusInEvent(self, evt)
         self.parent.formatChanged(None)
         self.parent.enableButtons()
+        if sys.platform.startswith("win32"):
+            self._programLayout = GetKeyboardLayout(0)
+            if self._ownLayout == None:
+                self._ownLayout = self._programLayout
+            ActivateKeyboardLayout(self._ownLayout, 0)
 
 class PreviewDialog(QDialog):
 
@@ -1067,6 +1087,11 @@ class PreviewDialog(QDialog):
         self.currentCard = 0
         self.dialog = ankiqt.forms.previewcards.Ui_Dialog()
         self.dialog.setupUi(self)
+        self.dialog.webView.page().setLinkDelegationPolicy(
+            QWebPage.DelegateExternalLinks)
+        self.connect(self.dialog.webView,
+                     SIGNAL("linkClicked(QUrl)"),
+                     self.linkClicked)
         self.dialog.comboBox.addItems(QStringList(
             [c.cardModel.name for c in self.cards]))
         self.connect(self.dialog.comboBox, SIGNAL("activated(int)"),
@@ -1075,16 +1100,24 @@ class PreviewDialog(QDialog):
         restoreGeom(self, "preview")
         self.exec_()
 
+    def linkClicked(self, url):
+        QDesktopServices.openUrl(QUrl(url))
+
     def updateCard(self):
         c = self.cards[self.currentCard]
+        styles = (self.deck.css +
+                  ("\nhtml { background: %s }" % c.cardModel.lastFontColour) +
+                  "\ndiv { white-space: pre-wrap; }")
+        styles = runFilter("addStyles", styles, c)
         self.dialog.webView.setHtml(
-            ('<html><head>%s</head><body>' % getBase(self.deck)) +
-            "<style>" + self.deck.css +
-            ("\nhtml { background: %s }" % c.cardModel.lastFontColour) +
-            "\ndiv { white-space: pre-wrap; }</style>" +
-            mungeQA(self.deck, c.htmlQuestion()) +
+            ('<html><head>%s</head><body>' % getBase(self.deck, c)) +
+            "<style>" + styles + "</style>" +
+            runFilter("drawQuestion", mungeQA(self.deck, c.htmlQuestion()),
+                      c) +
             "<br><br><hr><br><br>" +
-            mungeQA(self.deck, c.htmlAnswer()) + "</body></html>")
+            runFilter("drawAnswer", mungeQA(self.deck, c.htmlAnswer()),
+                      c)
+            + "</body></html>")
         playFromText(c.question)
         playFromText(c.answer)
 
