@@ -6,6 +6,7 @@ from PyQt4.QtGui import *
 from PyQt4.QtCore import *
 import anki
 import anki.importing as importing
+from ankiqt.ui.utils import getOnlyText
 from anki.errors import *
 import ankiqt.forms
 from ankiqt import ui
@@ -18,16 +19,21 @@ class ChangeMap(QDialog):
         self.dialog = ankiqt.forms.changemap.Ui_ChangeMap()
         self.dialog.setupUi(self)
         n = 0
+        setCurrent = False
         for field in self.model.fieldModels:
             item = QListWidgetItem(_("Map to %s") % field.name)
             self.dialog.fields.addItem(item)
-            if current == field.name:
+            if current and current.name == field.name:
+                setCurrent = True
                 self.dialog.fields.setCurrentRow(n)
             n += 1
         self.dialog.fields.addItem(QListWidgetItem(_("Map to Tags")))
         self.dialog.fields.addItem(QListWidgetItem(_("Discard field")))
-        if current is None:
-            self.dialog.fields.setCurrentRow(n)
+        if not setCurrent:
+            if current == 0:
+                self.dialog.fields.setCurrentRow(n)
+            else:
+                self.dialog.fields.setCurrentRow(n+1)
         self.field = None
 
     def getField(self):
@@ -54,6 +60,8 @@ class ImportDialog(QDialog):
         self.tags = ui.tagedit.TagEdit(parent)
         self.tags.setDeck(parent.deck)
         self.dialog.topGrid.addWidget(self.tags,0,1,1,1)
+        self.setTabOrder(self.tags, self.dialog.tagDuplicates)
+        self.setTabOrder(self.dialog.tagDuplicates, self.dialog.autoDetect)
         self.setupMappingFrame()
         self.setupOptions()
         self.getFile()
@@ -61,6 +69,9 @@ class ImportDialog(QDialog):
             return
         self.dialog.groupBox.setTitle(os.path.basename(self.file))
         self.maybePreview()
+        self.connect(self.dialog.autoDetect, SIGNAL("clicked()"),
+                     self.onDelimiter)
+        self.updateDelimiterButtonText()
         self.exec_()
 
     def setupOptions(self):
@@ -96,6 +107,7 @@ class ImportDialog(QDialog):
         else:
             self.modelChooser.hide()
             self.dialog.tagDuplicates.hide()
+        self.dialog.autoDetect.setShown(self.importerFunc.needDelimiter)
 
     def maybePreview(self):
         if self.file and self.model:
@@ -108,6 +120,45 @@ class ImportDialog(QDialog):
         self.model = model
         self.maybePreview()
 
+    def onDelimiter(self):
+        str = getOnlyText(_("""\
+By default, Anki will detect the character between fields, such as
+a tab, comma, and so on. If Anki is detecting the character incorrectly,
+you can enter it here. Use \\t to represent tab."""),
+                self, help="FileImport")
+        str = str.replace("\\t", "\t")
+        str = str.encode("ascii")
+        self.hideMapping()
+        def updateDelim():
+            self.importer.delimiter = str
+        self.showMapping(hook=updateDelim)
+        self.updateDelimiterButtonText()
+
+    def updateDelimiterButtonText(self):
+        if not self.importerFunc.needDelimiter:
+            return
+        if self.importer.delimiter:
+            d = self.importer.delimiter
+        else:
+            d = self.importer.dialect.delimiter
+        if d == "\t":
+            d = "Tab"
+        elif d == ",":
+            d = "Comma"
+        elif d == " ":
+            d = "Space"
+        elif d == ";":
+            d = "Semicolon"
+        elif d == ":":
+            d = "Colon"
+        else:
+            d = `d`
+        if self.importer.delimiter:
+            txt = _("Manual &delimiter: %s") % d
+        else:
+            txt = _("Auto-detected &delimiter: %s") % d
+        self.dialog.autoDetect.setText(txt)
+
     def doImport(self):
         self.dialog.status.setText(_("Importing..."))
         t = time.time()
@@ -126,7 +177,7 @@ class ImportDialog(QDialog):
                 return
             except Exception, e:
                 msg = _("Import failed.\n")
-                msg += traceback.format_exc()
+                msg += unicode(traceback.format_exc(), "ascii", "replace")
                 self.dialog.status.setText(msg)
                 return
         finally:
@@ -147,6 +198,7 @@ class ImportDialog(QDialog):
             # this fixes a strange bug in sqlite
             self.parent.deck.s.all("pragma integrity_check")
         self.parent.reset()
+        self.modelChooser.deinit()
 
     def setupMappingFrame(self):
         # qt seems to have a bug with adding/removing from a grid, so we add
@@ -157,10 +209,12 @@ class ImportDialog(QDialog):
     def hideMapping(self):
         self.dialog.mappingGroup.hide()
 
-    def showMapping(self, keepMapping=False):
+    def showMapping(self, keepMapping=False, hook=None):
         # first, check that we can read the file
         try:
             self.importer = self.importerFunc(self.parent.deck, self.file)
+            if hook:
+                hook()
             if not keepMapping:
                 self.mapping = self.importer.mapping
         except ImportFormatError, e:
@@ -203,6 +257,7 @@ class ImportDialog(QDialog):
             self.grid.addWidget(button, num, 2)
             self.connect(button, SIGNAL("clicked()"),
                          lambda s=self,n=num: s.changeMappingNum(n))
+        self.tags.setFocus()
 
     def changeMappingNum(self, n):
         f = ChangeMap(self.parent, self.model, self.mapping[n]).getField()
@@ -213,4 +268,14 @@ class ImportDialog(QDialog):
         except ValueError:
             pass
         self.mapping[n] = f
-        self.showMapping(keepMapping=True)
+        if getattr(self.importer, "delimiter", False):
+            self.savedDelimiter = self.importer.delimiter
+            def updateDelim():
+                self.importer.delimiter = self.savedDelimiter
+            self.showMapping(hook=updateDelim, keepMapping=True)
+        else:
+            self.showMapping(keepMapping=True)
+
+    def reject(self):
+        self.modelChooser.deinit()
+        QDialog.reject(self)
