@@ -5,10 +5,11 @@
 from PyQt4.QtGui import *
 from PyQt4.QtCore import *
 import anki, anki.utils
-from anki.sound import playFromText, stripSounds
+from anki.sound import playFromText
 from anki.latex import renderLatex, stripLatex
 from anki.utils import stripHTML
-from anki.hooks import runHook
+from anki.hooks import runHook, runFilter
+from anki.media import stripMedia
 import types, time, re, os, urllib, sys, difflib
 from ankiqt import ui
 from ankiqt.ui.utils import mungeQA, getBase
@@ -52,7 +53,7 @@ class View(object):
         "Idempotently display the current state (prompt for question, etc)"
         if self.state == "noDeck" or self.state == "studyScreen":
             return
-        self.clearWindow()
+        self.buffer = ""
         self.haveTop = (self.main.lastCard and (
             self.main.config['showLastCardContent'] or
             self.main.config['showLastCardInterval'])) or (
@@ -86,7 +87,8 @@ class View(object):
         s = "<style>\n"
         if self.main.deck:
             s += self.main.deck.css
-        s += "div { white-space: pre-wrap; }"
+        s += "div { white-space: pre-wrap; }\n"
+        s = runFilter("addStyles", s, self.main.currentCard)
         s += "</style>"
         return s
 
@@ -107,7 +109,7 @@ class View(object):
         # hook for user css
         runHook("preFlushHook")
         self.buffer = '''<html><head>%s</head><body>%s</body></html>''' % (
-            getBase(self.main.deck), self.buffer)
+            getBase(self.main.deck, self.main.currentCard), self.buffer)
         #print self.buffer.encode("utf-8")
         self.body.setHtml(self.buffer)
 
@@ -134,8 +136,11 @@ class View(object):
         q = self.main.currentCard.htmlQuestion()
         if self.haveTop:
             height = 35
+        elif self.main.currentCard.cardModel.questionInAnswer:
+            height = 40
         else:
             height = 45
+        q = runFilter("drawQuestion", q, self.main.currentCard)
         self.write(self.center(self.mungeQA(self.main.deck, q), height))
         if self.state != self.oldState and not nosound:
             playFromText(q)
@@ -148,10 +153,10 @@ class View(object):
         s = difflib.SequenceMatcher(None, b, a)
 
         sz = self.main.currentCard.cardModel.answerFontSize
-        ok = "background: %s; color: #000; font-size: %dpx" % (
-            passedCharColour, sz)
-        bad = "background: %s; color: #000; font-size: %dpx;" % (
-            failedCharColour, sz)
+        fn = self.main.currentCard.cardModel.answerFontFamily
+        st = "background: %s; color: #000; font-size: %dpx; font-family: %s;"
+        ok = st % (passedCharColour, sz, fn)
+        bad = st % (failedCharColour, sz, fn)
 
         for tag, i1, i2, j1, j2 in s.get_opcodes():
             if tag == "equal":
@@ -168,11 +173,13 @@ class View(object):
     def drawAnswer(self):
         "Show the answer."
         a = self.main.currentCard.htmlAnswer()
+        a = runFilter("drawAnswer", a, self.main.currentCard)
         if self.main.currentCard.cardModel.typeAnswer:
             try:
-                cor = stripHTML(self.main.currentCard.fact[
-                    self.main.currentCard.cardModel.typeAnswer])
+                cor = stripMedia(stripHTML(self.main.currentCard.fact[
+                    self.main.currentCard.cardModel.typeAnswer]))
             except KeyError:
+                self.main.currentCard.cardModel.typeAnswer = ""
                 cor = ""
             if cor:
                 given = unicode(self.main.typeAnswerField.text())
@@ -190,7 +197,7 @@ class View(object):
             txt = txt.replace("</span>", "&#8203;</span>")
         return txt
 
-    def onLoadFinished(self):
+    def onLoadFinished(self, bool):
         if self.state == "showAnswer":
             if self.main.config['scrollToAnswer']:
                 mf = self.body.page().mainFrame()
@@ -220,7 +227,7 @@ class View(object):
             return
         self.write("<span style='color: %s'>" % futureWarningColour +
                    _("This card was due in %s.") % fmtTimeSpan(
-            self.main.currentCard.due - time.time()) +
+            self.main.currentCard.due - time.time(), after=True) +
                    "</span>")
 
     def drawLastCard(self):
@@ -272,34 +279,19 @@ class View(object):
 <br>
 <table>
 
-<tr>
-<td>
-<a href="welcome:open"><img src=":/icons/document-open.png"></a>
-</td>
-<td valign=middle><h2><a href="welcome:open">%(local)s</a></h2></td>
-</tr>
-
 <tr>
 <td width=50>
-<a href="welcome:sample"><img src=":/icons/anki.png"></a>
+<a href="welcome:back"><img src=":/icons/go-previous.png"></a>
 </td>
-<td valign=middle><h2><a href="welcome:sample">%(dl_shared)s</a></h2></td>
-</tr>
-
-<tr>
-<td>
-<a href="welcome:openrem"><img src=":/icons/document-open-remote.png"></a>
-</td>
-<td valign=middle><h2><a href="welcome:openrem">%(dl_personal)s</a></h2></td>
+<td valign=middle><h2><a href="welcome:back">%(back)s</a></h2></td>
 </tr>
 
 </table>""" % \
 	{"welcome":_("Welcome to Anki!"),
-         "add":_("Add material"),
+         "add":_("Add Material"),
          "start":_("Start adding your own material."),
-         "local":_("Open Local Deck"),
-         "dl_shared":_("Download Shared Deck"),
-         "dl_personal":_("Download Personal Deck")})
+         "back":_("Back to Deck Browser"),
+         })
 
     def drawDeckFinishedMessage(self):
         "Tell the user the deck is finished."
