@@ -8,15 +8,16 @@ Exporting support
 """
 __docformat__ = 'restructuredtext'
 
-import itertools, time, re
+import itertools, time, re, os
 from operator import itemgetter
 from anki import DeckStorage
 from anki.cards import Card
-from anki.sync import SyncClient, SyncServer, BulkMediaSyncer
+from anki.sync import SyncClient, SyncServer, copyLocalMedia
 from anki.lang import _
 from anki.utils import findTag, parseTags, stripHTML, ids2str
 from anki.tags import tagIds
 from anki.db import *
+from BeautifulSoup import BeautifulSoup as BS
 
 class Exporter(object):
     def __init__(self, deck):
@@ -25,15 +26,25 @@ class Exporter(object):
         self.limitCardIds = []
 
     def exportInto(self, path):
+        self._escapeCount = 0
         file = open(path, "wb")
         self.doExport(file)
         file.close()
 
-    def escapeText(self, text):
+    def escapeText(self, text, removeFields=False):
         "Escape newlines and tabs, and strip Anki HTML."
         text = text.replace("\n", "<br>")
         text = text.replace("\t", " " * 8)
-        text = re.sub('<span class="fm.*?">(.*?)</span>', '\\1', text)
+        if removeFields:
+            # beautifulsoup is slow
+            self._escapeCount += 1
+            if self._escapeCount % 100 == 0:
+                self.deck.updateProgress()
+            s = BS(text)
+            all = s('span', {'class': re.compile("fm.*")})
+            for e in all:
+                e.replaceWith("".join([unicode(x) for x in e.contents]))
+            text = unicode(s)
         return text
 
     def cardIds(self):
@@ -43,7 +54,7 @@ class Exporter(object):
         if not self.limitTags:
             cards = self.deck.s.column0("select id from cards")
         else:
-            d = tagIds(self.deck.s, self.limitTags)
+            d = tagIds(self.deck.s, self.limitTags, create=False)
             cards = self.deck.s.column0(
                 "select cardId from cardTags where tagid in %s" %
                 ids2str(d.values()))
@@ -52,12 +63,13 @@ class Exporter(object):
 
 class AnkiExporter(Exporter):
 
-    key = _("Anki decks (*.anki)")
+    key = _("Anki Deck (*.anki)")
     ext = ".anki"
 
     def __init__(self, deck):
         Exporter.__init__(self, deck)
         self.includeSchedulingInfo = False
+        self.includeMedia = True
 
     def exportInto(self, path):
         n = 3
@@ -65,9 +77,14 @@ class AnkiExporter(Exporter):
             n += 1
         self.deck.startProgress(n)
         self.deck.updateProgress(_("Exporting..."))
+        try:
+            os.unlink(path)
+        except (IOError, OSError):
+            pass
         self.newDeck = DeckStorage.Deck(path)
         client = SyncClient(self.deck)
         server = SyncServer(self.newDeck)
+        server._mediaSupported = self.includeMedia
         client.setServer(server)
         client.localTime = self.deck.modified
         client.remoteTime = 0
@@ -116,11 +133,7 @@ modified = :now
             self.newDeck.s.statement("""
 delete from stats""")
         # media
-        if client.mediaSyncPending:
-            bulkClient = BulkMediaSyncer(client.deck)
-            bulkServer = BulkMediaSyncer(server.deck)
-            bulkClient.server = bulkServer
-            bulkClient.sync()
+        copyLocalMedia(client.deck, server.deck)
         # need to save manually
         self.newDeck.rebuildCounts()
         self.newDeck.updateAllPriorities()
@@ -171,22 +184,30 @@ class TextCardExporter(Exporter):
         self.includeTags = False
 
     def doExport(self, file):
-        strids = ids2str(self.cardIds())
+        ids = self.cardIds()
+        strids = ids2str(ids)
+        self.deck.startProgress((len(ids) + 1) / 50)
+        self.deck.updateProgress(_("Exporting..."))
         cards = self.deck.s.all("""
 select cards.question, cards.answer, cards.id from cards
-where cards.id in %s""" % strids)
+where cards.id in %s
+order by cards.created""" % strids)
+        self.deck.updateProgress()
         if self.includeTags:
             self.cardTags = dict(self.deck.s.all("""
-select cards.id, cards.tags || "," || facts.tags from cards, facts
+select cards.id, facts.tags from cards, facts
 where cards.factId = facts.id
-and cards.id in %s""" % strids))
-        out = u"\n".join(["%s\t%s%s" % (self.escapeText(c[0]),
-                                        self.escapeText(c[1]),
-                                        self.tags(c[2]))
+and cards.id in %s
+order by cards.created""" % strids))
+        out = u"\n".join(["%s\t%s%s" % (
+            self.escapeText(c[0], removeFields=True),
+            self.escapeText(c[1], removeFields=True),
+            self.tags(c[2]))
                           for c in cards])
         if out:
             out += "\n"
         file.write(out.encode("utf-8"))
+        self.deck.finishProgress()
 
     def tags(self, id):
         if self.includeTags:
@@ -204,25 +225,35 @@ class TextFactExporter(Exporter):
 
     def doExport(self, file):
         cardIds = self.cardIds()
+        self.deck.startProgress()
+        self.deck.updateProgress(_("Exporting..."))
         facts = self.deck.s.all("""
-select factId, value from fields
+select factId, value, facts.created from facts, fields
 where
-factId in
-(select distinct facts.id from facts, cards
-where facts.id = cards.factId
-and cards.id in %s)
+facts.id in
+(select distinct factId from cards
+where cards.id in %s)
+and facts.id = fields.factId
 order by factId, ordinal""" % ids2str(cardIds))
         txt = ""
+        self.deck.updateProgress()
         if self.includeTags:
             self.factTags = dict(self.deck.s.all(
                 "select id, tags from facts where id in %s" %
                 ids2str([fact[0] for fact in facts])))
-        out = ["\t".join([self.escapeText(x[1]) for x in ret[1]]) +
-               self.tags(ret[0])
-               for ret in (itertools.groupby(facts, itemgetter(0)))]
+        groups = itertools.groupby(facts, itemgetter(0))
+        groups = [[x for x in y[1]] for y in groups]
+        groups = [(group[0][2],
+                   "\t".join([self.escapeText(x[1]) for x in group]) +
+                   self.tags(group[0][0]))
+                  for group in groups]
+        self.deck.updateProgress()
+        groups.sort(key=itemgetter(0))
+        out = [ret[1] for ret in groups]
         self.count = len(out)
         out = "\n".join(out)
         file.write(out.encode("utf-8"))
+        self.deck.finishProgress()
 
     def tags(self, id):
         if self.includeTags:
@@ -234,6 +265,6 @@ order by factId, ordinal""" % ids2str(cardIds))
 
 def exporters():
     return (
-        (_("Anki deck (*.anki)"), AnkiExporter),
+        (_("Anki Deck (*.anki)"), AnkiExporter),
         (_("Cards in tab-separated text file (*.txt)"), TextCardExporter),
         (_("Facts in tab-separated text file (*.txt)"), TextFactExporter))
