@@ -36,6 +36,7 @@ class Importer(object):
     needMapper = True
     tagDuplicates = False
     multipleCardsAllowed = True
+    needDelimiter = False
 
     def __init__(self, deck, file):
         self.file = file
@@ -121,9 +122,9 @@ class Importer(object):
             if cm.active: active += 1
         if active > 1 and not self.multipleCardsAllowed:
             raise ImportFormatError(type="tooManyCards",
-                info=_("""
-The current importer only supports a single active card template. Please disable
-all but one card template."""))
+                info=_("""\
+The current importer only supports a single active card template. Please disable\
+ all but one card template."""))
         # strip invalid cards
         cards = self.stripInvalid(cards)
         cards = self.stripOrTagDupes(cards)
@@ -144,10 +145,17 @@ all but one card template."""))
         # add facts
         self.deck.updateProgress()
         factIds = [genID() for n in range(len(cards))]
+        def fudgeCreated(d, tmp=[]):
+            if not tmp:
+                tmp.append(time.time())
+            else:
+                tmp[0] += 0.00001
+            d['created'] = tmp[0]
+            return d
         self.deck.s.execute(factsTable.insert(),
-            [{'modelId': self.model.id,
+            [fudgeCreated({'modelId': self.model.id,
               'tags': canonifyTags(self.tagsToAdd + " " + cards[n].tags),
-              'id': factIds[n]} for n in range(len(cards))])
+              'id': factIds[n]}) for n in range(len(cards))])
         self.deck.factCount += len(factIds)
         self.deck.s.execute("""
 delete from factsDeleted
@@ -171,9 +179,11 @@ where factId in (%s)""" % ",".join([str(s) for s in factIds]))
         # and cards
         self.deck.updateProgress()
         now = time.time()
+        active = 0
         for cm in self.model.cardModels:
             self._now = now
             if cm.active:
+                active += 1
                 data = [self.addMeta({
                     'id': genID(),
                     'factId': factIds[m],
@@ -186,18 +196,20 @@ where factId in (%s)""" % ",".join([str(s) for s in factIds]))
                                     data)
         self.deck.updateProgress()
         self.deck.updateCardsFromFactIds(factIds)
-        self.deck.cardCount += len(cards)
+        self.deck.cardCount += len(cards) * active
         self.total = len(factIds)
 
     def addMeta(self, data, card):
         "Add any scheduling metadata to cards"
         if 'fields' in card.__dict__:
             del card.fields
-        data['created'] = self._now
-        data['modified'] = self._now
-        data['due'] = self._now
+        t = self._now + data['ordinal']
+        data['created'] = t
+        data['modified'] = t
+        data['due'] = t
         self._now += .00001
         data.update(card.__dict__)
+        data['tags'] = u""
         self.cardIds.append(data['id'])
         data['combinedDue'] = data['due']
         data['isDue'] = data['combinedDue'] < time.time()
@@ -232,7 +244,7 @@ where factId in (%s)""" % ",".join([str(s) for s in factIds]))
             fmid=field.id))
 
     def cardIsUnique(self, card):
-        fields = []
+        fieldsAsTags = []
         for n in range(len(self.mapping)):
             if self.mapping[n] and self.mapping[n].unique:
                 if card.fields[n] in self.uniqueCache[self.mapping[n].id]:
@@ -241,26 +253,30 @@ where factId in (%s)""" % ",".join([str(s) for s in factIds]))
                                         (self.mapping[n].name,
                                          ", ".join(card.fields)))
                         return False
-                    fields.append(self.mapping[n].name)
+                    fieldsAsTags.append(self.mapping[n].name.replace(" ", "-"))
                 else:
                     self.uniqueCache[self.mapping[n].id][card.fields[n]] = 1
-        if fields:
+        if fieldsAsTags:
             card.tags += u" Duplicate:" + (
-                "+".join(fields))
+                "+".join(fieldsAsTags))
             card.tags = canonifyTags(card.tags)
         return True
 
 # Export modules
 ##########################################################################
 
-from anki.importing.csv import TextImporter
+from anki.importing.csvfile import TextImporter
 from anki.importing.anki10 import Anki10Importer
 from anki.importing.mnemosyne10 import Mnemosyne10Importer
 from anki.importing.wcu import WCUImporter
+from anki.importing.supermemo_xml import SupermemoXmlImporter
+from anki.importing.dingsbums import DingsBumsImporter
 
 Importers = (
     (_("Text separated by tabs or semicolons (*)"), TextImporter),
     (_("Anki Deck (*.anki)"), Anki10Importer),
     (_("Mnemosyne Deck (*.mem)"), Mnemosyne10Importer),
     (_("CueCard Deck (*.wcu)"), WCUImporter),
+    (_("Supermemo XML export (*.xml)"), SupermemoXmlImporter),
+    (_("DingsBums?! Deck (*.xml)"), DingsBumsImporter),
     )
