@@ -8,9 +8,8 @@ Latex support
 """
 __docformat__ = 'restructuredtext'
 
-import re, tempfile, os, sys, subprocess, stat, time
+import re, tempfile, os, sys, subprocess, stat, time, shutil
 from anki.utils import genID, checksum
-from anki.media import copyToMedia
 from htmlentitydefs import entitydefs
 
 latexPreamble = ("\\documentclass[12pt]{article}\n"
@@ -77,13 +76,7 @@ def call(argv, wait=True, **kwargs):
 
 def latexImgFile(deck, latexCode):
     key = checksum(latexCode)
-    return deck.s.scalar("select filename from media where originalPath = :k",
-                         k=key)
-
-def latexImgPath(deck, file):
-    "Return the path to the cache file in system encoding format."
-    path = os.path.join(deck.mediaDir(create=True), file)
-    return path.encode(sys.getfilesystemencoding())
+    return "latex-%s.png" % key
 
 def mungeLatex(latex):
     "Convert entities, fix newlines, and convert to utf8."
@@ -95,20 +88,25 @@ def mungeLatex(latex):
     return latex
 
 def deleteAllLatexImages(deck):
-    for f in deck.s.column0(
-        "select filename from media where description = 'latex'"):
-        path = latexImgPath(deck, f)
-        try:
-            os.unlink(path)
-        except (OSError, IOError):
-            pass
-    deck.s.statement("delete from media where description = 'latex'")
-    deck.flushMod()
+    mdir = deck.mediaDir()
+    if not mdir:
+        return
+    deck.startProgress()
+    for c, f in enumerate(os.listdir(mdir)):
+        if f.startswith("latex-"):
+            os.unlink(os.path.join(mdir, f))
+        if c % 100 == 0:
+            deck.updateProgress()
+    deck.finishProgress()
 
 def cacheAllLatexImages(deck):
+    deck.startProgress()
     fields = deck.s.column0("select value from fields")
-    for field in fields:
+    for c, field in enumerate(fields):
+        if c % 10 == 0:
+            deck.updateProgress()
         renderLatex(deck, field)
+    deck.finishProgress()
 
 def buildImg(deck, latex):
     log = open(os.path.join(tmpdir, "latex_log.txt"), "w+")
@@ -119,6 +117,8 @@ def buildImg(deck, latex):
     texfile.write(latexPostamble + "\n")
     texfile.close()
     texpath = texpath.encode(sys.getfilesystemencoding())
+    # make sure we have a valid mediaDir
+    deck.mediaDir(create=True)
     oldcwd = os.getcwd()
     if sys.platform == "win32":
         si = subprocess.STARTUPINFO()
@@ -127,28 +127,26 @@ def buildImg(deck, latex):
         si = None
     try:
         os.chdir(tmpdir)
-        errmsg = _(
-            "Error executing 'latex' or 'dvipng'.\n"
+        errmsg = _("Error executing %s.\n") + _(
             "A log file is available here:\n%s") % tmpdir
         if call(["latex", "-interaction=nonstopmode",
                  texpath], stdout=log, stderr=log, startupinfo=si):
-            return (False, errmsg)
+            return (False, errmsg % "latex")
         if call(latexDviPngCmd + ["tmp.dvi", "-o", "tmp.png"],
                 stdout=log, stderr=log, startupinfo=si):
-            return (False, errmsg)
+            return (False, errmsg % "dvipng")
         # add to media
-        path = copyToMedia(deck, "tmp.png", latex=checksum(latex))
-        return (True, path)
+        target = latexImgFile(deck, latex)
+        shutil.copy2("tmp.png", os.path.join(deck.mediaDir(), target))
+        return (True, target)
     finally:
         os.chdir(oldcwd)
 
 def imageForLatex(deck, latex, build=True):
     "Return an image that represents 'latex', building if necessary."
     imageFile = latexImgFile(deck, latex)
-    if imageFile:
-        path = latexImgPath(deck, imageFile)
     ok = True
-    if build and (not imageFile or not os.path.exists(path)):
+    if build and (not imageFile or not os.path.exists(imageFile)):
         (ok, imageFile) = buildImg(deck, latex)
     if not ok:
         return (False, imageFile)
