@@ -18,6 +18,7 @@ from datetime import date
 from anki.db import *
 from anki.lang import _
 from anki.utils import canonifyTags, ids2str
+from anki.hooks import runFilter
 
 # Tracking stats on the DB
 ##########################################################################
@@ -153,8 +154,7 @@ def updateStats(s, stats, card, ease, oldState):
     stats.reps += 1
     delay = card.totalTime()
     if delay >= 60:
-        # make a guess as to the time spent answering
-        stats.reviewTime += stats.averageTime
+        stats.reviewTime += 60
     else:
         stats.reviewTime += delay
         stats.averageTime = (
@@ -266,12 +266,13 @@ class CardStats(object):
         if c.firstAnswered:
             self.addLine(_("First Review"), self.strTime(c.firstAnswered))
         self.addLine(_("Changed"), self.strTime(c.modified))
-        next = time.time() - c.due
-        if next > 0:
-            next = _("%s ago") % fmt(next)
-        else:
-            next = _("in %s") % fmt(abs(next))
-        self.addLine(_("Due"), next)
+        if c.reps:
+            next = time.time() - c.combinedDue
+            if next > 0:
+                next = _("%s ago") % fmt(next)
+            else:
+                next = _("in %s") % fmt(abs(next))
+            self.addLine(_("Due"), next)
         self.addLine(_("Interval"), fmt(c.interval * 86400))
         self.addLine(_("Ease"), fmtFloat(c.factor, point=2))
         if c.lastDue:
@@ -331,66 +332,178 @@ class DeckStats(object):
         html += _("Total number of cards:") + " <b>%d</b><br>" % total
         html += _("Total number of facts:") + " <b>%d</b><br><br>" % d.factCount
 
-        html += "<b>" + _("Card counts") + "</b><br>"
+        html += "<b>" + _("Card Maturity") + "</b><br>"
         html += _("Mature cards: <!--card count-->") + " <b>%(old)d</b> (%(oldP)s)<br>" % {
                 'old': stats['old'], 'oldP' : fmtPerc(stats['oldP'])}
         html += _("Young cards: <!--card count-->") + " <b>%(young)d</b> (%(youngP)s)<br>" % {
                 'young': stats['young'], 'youngP' : fmtPerc(stats['youngP'])}
-        html += _("Unseen cards:") + " <b>%(new)d</b> (%(newP)s)<br><br>" % {
+        html += _("Unseen cards:") + " <b>%(new)d</b> (%(newP)s)<br>" % {
                 'new': stats['new'], 'newP' : fmtPerc(stats['newP'])}
-        html += "<b>" + _("Correct answers") + "</b><br>"
+        avgInt = self.getAverageInterval()
+        if avgInt:
+            html += _("Average interval: ") + ("<b>%s</b> ") % fmtFloat(avgInt) + _("days")
+            html += "<br>"
+        html += "<br>"
+        inactive = d.inactiveCardCount()
+        suspended = d.suspendedCardCount()
+        active = total - inactive - suspended
+        inactiveP = inactive / float(total) * 100
+        suspendedP = suspended / float(total) * 100
+        activeP = active / float(total) * 100
+        html += "<b>" + _("Card State") + "</b><br>"
+        html += _("Active cards:") + " <b>%(a)d</b> (%(b)s)<br>" % {
+            'a': active, 'b' : fmtPerc(activeP)}
+        html += _("Inactive cards:") + " <b>%(a)d</b> (%(b)s)<br>" % {
+            'a': inactive, 'b' : fmtPerc(inactiveP)}
+        html += _("Suspended cards:") + " <b>%(a)d</b> (%(b)s)<br><br>" % {
+            'a': suspended, 'b' : fmtPerc(suspendedP)}
+        html += "<b>" + _("Correct Answers") + "</b><br>"
         html += _("Mature cards: <!--correct answers-->") + " <b>" + fmtPerc(stats['gMatureYes%']) + (
-                "</b> " + _("%(partOf)d of %(totalSum)d") % {
+                "</b> " + _("(%(partOf)d of %(totalSum)d)") % {
                 'partOf' : stats['gMatureYes'],
                 'totalSum' : stats['gMatureTotal'] } + "<br>")
         html += _("Young cards: <!--correct answers-->")  + " <b>" + fmtPerc(stats['gYoungYes%']) + (
-                "</b> " + _("%(partOf)d of %(totalSum)d") % {
+                "</b> " + _("(%(partOf)d of %(totalSum)d)") % {
                 'partOf' : stats['gYoungYes'],
                 'totalSum' : stats['gYoungTotal'] } + "<br>")
         html += _("First-seen cards:") + " <b>" + fmtPerc(stats['gNewYes%']) + (
-                "</b> " + _("%(partOf)d of %(totalSum)d") % {
+                "</b> " + _("(%(partOf)d of %(totalSum)d)") % {
                 'partOf' : stats['gNewYes'],
                 'totalSum' : stats['gNewTotal'] } + "<br><br>")
 
         # average pending time
         existing = d.cardCount - d.newCountToday
-        avgInt = self.getAverageInterval()
         def tr(a, b):
             return "<tr><td>%s</td><td align=right>%s</td></tr>" % (a, b)
         if existing and avgInt:
-            html += "<b>" + _("Averages") + "</b>"
+            html += "<b>" + _("Recent Work") + "</b>"
             if sys.platform.startswith("darwin"):
                 html += "<table width=250>"
             else:
                 html += "<table width=200>"
-            html += tr(_("Interval"), ("<b>%s</b> ") % fmtFloat(avgInt) + _("days") )
-            html += tr(_("Average reps"), ("<b>%s</b> ") % (
+            html += tr(_("In last week"),
+                       ("<b>%d</b> reps/<b>%d</b> days") % (
+                self.getRepsDone(-7, 0),
+                self.getDaysReviewed(-7, 0)))
+            html += tr(_("In last month"),
+                       ("<b>%d</b> reps/<b>%d</b> days") % (
+                self.getRepsDone(-30, 0),
+                self.getDaysReviewed(-30, 0)))
+            html += tr(_("In last 3 months"),
+                       ("<b>%d</b> reps/<b>%d</b> days") % (
+                self.getRepsDone(-92, 0),
+                self.getDaysReviewed(-92, 0)))
+            html += tr(_("In last 6 months"),
+                       ("<b>%d</b> reps/<b>%d</b> days") % (
+                self.getRepsDone(-182, 0),
+                self.getDaysReviewed(-182, 0)))
+            html += tr(_("In last year"),
+                       ("<b>%d</b> reps/<b>%d</b> days") % (
+                self.getRepsDone(-365, 0),
+                self.getDaysReviewed(-365, 0)))
+            html += tr(_("Deck life"),
+                       ("<b>%d</b> reps/<b>%d</b> days") % (
+                self.getRepsDone(-13000, 0),
+                self.getDaysReviewed(-13000, 0)))
+            html += "</table>"
+
+            html += "<br><br><b>" + _("Average Daily Reviews") + "</b>"
+            if sys.platform.startswith("darwin"):
+                html += "<table width=250>"
+            else:
+                html += "<table width=200>"
+            html += tr(_("Deck life"), ("<b>%s</b> ") % (
                 fmtFloat(self.getSumInverseRoundInterval())) + _("cards/day"))
-            html += tr(_("Reps next week"), ("<b>%s</b> ") % (
+            html += tr(_("In next week"), ("<b>%s</b> ") % (
                 fmtFloat(self.getWorkloadPeriod(7))) + _("cards/day"))
-            html += tr(_("Reps next month"), ("<b>%s</b> ") % (
+            html += tr(_("In next month"), ("<b>%s</b> ") % (
                 fmtFloat(self.getWorkloadPeriod(30))) + _("cards/day"))
-            html += tr(_("Reps last week"), ("<b>%s</b> ") % (
+            html += tr(_("In last week"), ("<b>%s</b> ") % (
                 fmtFloat(self.getPastWorkloadPeriod(7))) + _("cards/day"))
-            html += tr(_("Reps last month"), ("<b>%s</b> ") % (
+            html += tr(_("In last month"), ("<b>%s</b> ") % (
                 fmtFloat(self.getPastWorkloadPeriod(30))) + _("cards/day"))
-            html += tr(_("Avg. added"), _("<b>%(a)s</b>/day, <b>%(b)s</b>/mon") % {
+            html += tr(_("In last 3 months"), ("<b>%s</b> ") % (
+                fmtFloat(self.getPastWorkloadPeriod(92))) + _("cards/day"))
+            html += tr(_("In last 6 months"), ("<b>%s</b> ") % (
+                fmtFloat(self.getPastWorkloadPeriod(182))) + _("cards/day"))
+            html += tr(_("In last year"), ("<b>%s</b> ") % (
+                fmtFloat(self.getPastWorkloadPeriod(365))) + _("cards/day"))
+            html += "</table>"
+
+            html += "<br><br><b>" + _("Average Added") + "</b>"
+            if sys.platform.startswith("darwin"):
+                html += "<table width=250>"
+            else:
+                html += "<table width=200>"
+            html += tr(_("Deck life"), _("<b>%(a)s</b>/day, <b>%(b)s</b>/mon") % {
                 'a': fmtFloat(self.newAverage()), 'b': fmtFloat(self.newAverage()*30)})
             np = self.getNewPeriod(7)
-            html += tr(_("Added last week"), _("<b>%(a)d</b> (<b>%(b)s</b>/day)") % (
+            html += tr(_("In last week"), _("<b>%(a)d</b> (<b>%(b)s</b>/day)") % (
                 {'a': np, 'b': fmtFloat(np / float(7))}))
             np = self.getNewPeriod(30)
-            html += tr(_("Added last month"), _("<b>%(a)d</b> (<b>%(b)s</b>/day)") % (
+            html += tr(_("In last month"), _("<b>%(a)d</b> (<b>%(b)s</b>/day)") % (
                 {'a': np, 'b': fmtFloat(np / float(30))}))
+            np = self.getNewPeriod(92)
+            html += tr(_("In last 3 months"), _("<b>%(a)d</b> (<b>%(b)s</b>/day)") % (
+                {'a': np, 'b': fmtFloat(np / float(92))}))
+            np = self.getNewPeriod(182)
+            html += tr(_("In last 6 months"), _("<b>%(a)d</b> (<b>%(b)s</b>/day)") % (
+                {'a': np, 'b': fmtFloat(np / float(182))}))
+            np = self.getNewPeriod(365)
+            html += tr(_("In last year"), _("<b>%(a)d</b> (<b>%(b)s</b>/day)") % (
+                {'a': np, 'b': fmtFloat(np / float(365))}))
+            html += "</table>"
+
+            html += "<br><br><b>" + _("Average New Seen") + "</b>"
+            if sys.platform.startswith("darwin"):
+                html += "<table width=250>"
+            else:
+                html += "<table width=200>"
             np = self.getFirstPeriod(7)
-            html += tr(_("First last week"), _("<b>%(a)d</b> (<b>%(b)s</b>/day)") % (
+            html += tr(_("In last week"), _("<b>%(a)d</b> (<b>%(b)s</b>/day)") % (
                 {'a': np, 'b': fmtFloat(np / float(7))}))
             np = self.getFirstPeriod(30)
-            html += tr(_("First last month"), _("<b>%(a)d</b> (<b>%(b)s</b>/day)") % (
+            html += tr(_("In last month"), _("<b>%(a)d</b> (<b>%(b)s</b>/day)") % (
                 {'a': np, 'b': fmtFloat(np / float(30))}))
+            np = self.getFirstPeriod(92)
+            html += tr(_("In last 3 months"), _("<b>%(a)d</b> (<b>%(b)s</b>/day)") % (
+                {'a': np, 'b': fmtFloat(np / float(92))}))
+            np = self.getFirstPeriod(182)
+            html += tr(_("In last 6 months"), _("<b>%(a)d</b> (<b>%(b)s</b>/day)") % (
+                {'a': np, 'b': fmtFloat(np / float(182))}))
+            np = self.getFirstPeriod(365)
+            html += tr(_("In last year"), _("<b>%(a)d</b> (<b>%(b)s</b>/day)") % (
+                {'a': np, 'b': fmtFloat(np / float(365))}))
             html += "</table>"
+
+            html += "<br><br><b>" + _("Card Ease") + "</b><br>"
+            html += _("Lowest factor: %.2f") % d.s.scalar(
+                "select min(factor) from cards") + "<br>"
+            html += _("Average factor: %.2f") % d.s.scalar(
+                "select avg(factor) from cards") + "<br>"
+            html += _("Highest factor: %.2f") % d.s.scalar(
+                "select max(factor) from cards") + "<br>"
+
+            html = runFilter("deckStats", html)
         return html
 
+    def getDaysReviewed(self, start, finish):
+        now = datetime.datetime.today()
+        x = now + datetime.timedelta(start)
+        y = now + datetime.timedelta(finish)
+        return self.deck.s.scalar(
+            "select count() from stats where "
+            "day >= :x and day <= :y and reps > 0",
+            x=x, y=y)
+
+    def getRepsDone(self, start, finish):
+        now = datetime.datetime.today()
+        x = time.mktime((now + datetime.timedelta(start)).timetuple())
+        y = time.mktime((now + datetime.timedelta(finish)).timetuple())
+        return self.deck.s.scalar(
+            "select count() from reviewHistory where time >= :x and time <= :y",
+            x=x, y=y)
+
     def getAverageInterval(self):
         return self.deck.s.scalar(
             "select sum(interval) / count(interval) from cards "
@@ -460,153 +573,3 @@ where created > :cutoff""", cutoff=cutoff) or 0)
         return (self.deck.s.scalar("""
 select count(*) from reviewHistory
 where reps = 1 and time > :cutoff""", cutoff=cutoff) or 0)
-
-# Kanji stats
-##########################################################################
-
-def asHTMLDocument(text):
-    "Return text wrapped in a HTML document."
-    return ("""
-    <!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN"
-	"http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
-    <html>
-    <head>
-    <meta http-equiv=content-type content="text/html; charset=utf-8">
-    </head>
-    <body>
-    %s
-    </body>
-    </html>
-    """ % text.encode("utf-8"))
-
-def isKanji(unichar):
-    try:
-        return unicodedata.name(unichar).find('CJK UNIFIED IDEOGRAPH') >= 0
-    except ValueError:
-        # a control character
-        return False
-
-class KanjiStats(object):
-
-    def __init__(self, deck):
-        self.deck = deck
-        self._gradeHash = dict()
-        for (name, chars), grade in zip(self.kanjiGrades,
-                                        xrange(len(self.kanjiGrades))):
-            for c in chars:
-                self._gradeHash[c] = grade
-
-    def kanjiGrade(self, unichar):
-        return self._gradeHash.get(unichar, 0)
-
-    # FIXME: as it's html, the width doesn't matter
-    def kanjiCountStr(self, gradename, count, total=0, width=0):
-        d = {'count': self.rjustfig(count, width), 'gradename': gradename}
-        if total:
-            d['total'] = self.rjustfig(total, width)
-            d['percent'] = float(count)/total*100
-            return _("%(gradename)s: %(count)s of %(total)s (%(percent)0.1f%%).") % d
-        else:
-            return _("%(count)s %(gradename)s kanji.") % d
-
-    def rjustfig(self, n, width):
-        n = unicode(n)
-        return n + "&nbsp;" * (width - len(n))
-
-    def genKanjiSets(self):
-        self.kanjiSets = [set([]) for g in self.kanjiGrades]
-        mids = self.deck.s.column0('''
-select id from models where tags like "%Japanese%"''')
-        all = "".join(self.deck.s.column0("""
-select value from cards, fields, facts
-where
-cards.reps > 0 and
-cards.factId = fields.factId
-and cards.factId = facts.id
-and facts.modelId in %s
-""" % ids2str(mids)))
-        for u in all:
-            if isKanji(u):
-                self.kanjiSets[self.kanjiGrade(u)].add(u)
-
-    def report(self):
-        self.genKanjiSets()
-        counts = [(name, len(found), len(all)) \
-                  for (name, all), found in zip(self.kanjiGrades, self.kanjiSets)]
-        out = (_("<h1>Kanji statistics</h1>The %d seen cards in this deck "
-                 "contain:") % self.deck.seenCardCount() +
-               "<ul>" +
-               # total kanji
-               _("<li>%d total unique kanji.</li>") %
-               sum([c[1] for c in counts]) +
-               # total joyo
-               "<li>%s</li>" % self.kanjiCountStr(
-            u'Jouyou',sum([c[1] for c in counts[1:8]]),
-            sum([c[2] for c in counts[1:8]])) +
-               # total jinmei
-               "<li>%s</li>" % self.kanjiCountStr(*counts[8]) +
-               # total non-joyo
-               "<li>%s</li>" % self.kanjiCountStr(*counts[0]))
-
-        out += "</ul><p/>" + _(u"Jouyou levels:") + "<p/><ul>"
-        L = ["<li>" + self.kanjiCountStr(c[0],c[1],c[2], width=3) + "</li>"
-             for c in counts[1:8]]
-        out += "".join(L)
-        out += "</ul>"
-        return out
-
-    def missingReport(self, check=None):
-        if not check:
-            check = lambda x, y: x not in y
-            out = _("<h1>Missing</h1>")
-        else:
-            out = _("<h1>Seen</h1>")
-        for grade in range(1, 9):
-            missing = "".join(self.missingInGrade(grade, check))
-            if not missing:
-                continue
-            out += "<h2>" + self.kanjiGrades[grade][0] + "</h2>"
-            out += "<font size=+4>"
-            out += self.mkEdict(missing)
-            out += "</font>"
-        return out + "<br/>"
-
-    def mkEdict(self, kanji):
-        out = "<font size=+4>"
-        while 1:
-            if not kanji:
-                out += "</font>"
-                return out
-            # edict will take up to about 10 kanji at once
-            out += self.edictKanjiLink(kanji[0:10])
-            kanji = kanji[10:]
-
-    def seenReport(self):
-        return self.missingReport(lambda x, y: x in y)
-
-    def nonJouyouReport(self):
-        out = _("<h1>Non-Jouyou</h1>")
-        out += self.mkEdict("".join(self.kanjiSets[0]))
-        return out + "<br/>"
-
-    def edictKanjiLink(self, kanji):
-        base="http://www.csse.monash.edu.au/~jwb/cgi-bin/wwwjdic.cgi?1MMJ"
-        url=base + kanji
-        return '<a href="%s">%s</a>' % (url, kanji)
-
-    def missingInGrade(self, gradeNum, check):
-        existingKanji = self.kanjiSets[gradeNum]
-        totalKanji = self.kanjiGrades[gradeNum][1]
-        return [k for k in totalKanji if check(k, existingKanji)]
-
-    kanjiGrades = [
-        (u'non-jouyou', ''),
-        (u'Grade 1', u'一右雨円王音下火花貝学気休玉金九空月犬見五口校左三山四子糸字耳七車手十出女小上森人水正生青石赤先千川早草足村大男竹中虫町天田土二日入年白八百文本名木目夕立力林六'),
-        (u'Grade 2', u'引羽雲園遠黄何夏家科歌画会回海絵外角楽活間丸岩顔帰汽記弓牛魚京強教近兄形計元原言古戸午後語交光公工広考行高合国黒今才細作算姉市思止紙寺時自室社弱首秋週春書少場色食心新親図数星晴声西切雪線船前組走多太体台谷知地池茶昼朝長鳥直通弟店点電冬刀東当答頭同道読内南肉馬買売麦半番父風分聞米歩母方北妹毎万明鳴毛門夜野矢友曜用来理里話'),
-        (u'Grade 3', u'悪安暗委意医育員飲院運泳駅央横屋温化荷界開階寒感漢館岸期起客宮急球究級去橋業局曲銀区苦具君係軽決血研県庫湖向幸港号根祭坂皿仕使始指死詩歯事持次式実写者主取守酒受州拾終習集住重宿所暑助勝商昭消章乗植深申真神身進世整昔全想相送息速族他打対待代第題炭短談着柱注丁帳調追定庭笛鉄転登都度島投湯等豆動童農波配倍箱畑発反板悲皮美鼻筆氷表病秒品負部服福物平返勉放味命面問役薬油有由遊予様洋羊葉陽落流旅両緑礼列練路和'),
-        (u'Grade 4', u'愛案以位囲胃衣印栄英塩億加果課貨芽改械害街各覚完官管観関願喜器希旗機季紀議救求泣給挙漁競共協鏡極訓軍郡型径景芸欠結健建験固候功好康航告差最菜材昨刷察札殺参散産残司史士氏試児治辞失借種周祝順初唱松焼照省笑象賞信臣成清静席積折節説戦浅選然倉巣争側束続卒孫帯隊達単置仲貯兆腸低停底的典伝徒努灯働堂得特毒熱念敗梅博飯費飛必標票不付夫府副粉兵別変辺便包法望牧末満未脈民無約勇要養浴利陸料良量輪類令例冷歴連労老録'),
-        (u'Grade 5', u'圧易移因営永衛液益演往応恩仮価可河過賀解快格確額刊幹慣眼基寄規技義逆久旧居許境興均禁句群経潔件券検険減現限個故護効厚構耕講鉱混査再妻採災際在罪財桜雑賛酸師志支枝資飼似示識質舎謝授修術述準序承招証常情条状織職制勢性政精製税績責接設絶舌銭祖素総像増造則測属損態貸退団断築張提程敵適統導銅徳独任燃能破判版犯比肥非備俵評貧婦富布武復複仏編弁保墓報豊暴貿防務夢迷綿輸余預容率略留領'),
-        (u'Grade 6', u'異遺域宇映延沿我灰拡閣革割株巻干看簡危揮机貴疑吸供胸郷勤筋敬系警劇激穴憲権絹厳源呼己誤后孝皇紅鋼降刻穀骨困砂座済裁策冊蚕姿私至視詞誌磁射捨尺若樹収宗就衆従縦縮熟純処署諸除傷将障城蒸針仁垂推寸盛聖誠宣専泉洗染善創奏層操窓装臓蔵存尊宅担探誕暖段値宙忠著庁潮頂賃痛展党糖討届難乳認納脳派俳拝背肺班晩否批秘腹奮並閉陛片補暮宝訪亡忘棒枚幕密盟模訳優郵幼欲翌乱卵覧裏律臨朗論'),
-        (u'JuniorHS', u'亜哀握扱依偉威尉慰為維緯違井壱逸稲芋姻陰隠韻渦浦影詠鋭疫悦謁越閲宴援炎煙猿縁鉛汚凹奥押欧殴翁沖憶乙卸穏佳嫁寡暇架禍稼箇華菓蚊雅餓介塊壊怪悔懐戒拐皆劾慨概涯該垣嚇核殻獲穫較郭隔岳掛潟喝括渇滑褐轄且刈乾冠勘勧喚堪寛患憾換敢棺款歓汗環甘監緩缶肝艦貫還鑑閑陥含頑企奇岐幾忌既棋棄祈軌輝飢騎鬼偽儀宜戯擬欺犠菊吉喫詰却脚虐丘及朽窮糾巨拒拠虚距享凶叫峡恐恭挟況狂狭矯脅響驚仰凝暁斤琴緊菌襟謹吟駆愚虞偶遇隅屈掘靴繰桑勲薫傾刑啓契恵慶憩掲携渓継茎蛍鶏迎鯨撃傑倹兼剣圏堅嫌懸献肩謙賢軒遣顕幻弦玄孤弧枯誇雇顧鼓互呉娯御悟碁侯坑孔巧恒慌抗拘控攻更江洪溝甲硬稿絞綱肯荒衡貢購郊酵項香剛拷豪克酷獄腰込墾婚恨懇昆紺魂佐唆詐鎖債催宰彩栽歳砕斎載剤咲崎削搾索錯撮擦傘惨桟暫伺刺嗣施旨祉紫肢脂諮賜雌侍慈滋璽軸執湿漆疾芝赦斜煮遮蛇邪勺爵酌釈寂朱殊狩珠趣儒寿需囚愁秀臭舟襲酬醜充柔汁渋獣銃叔淑粛塾俊瞬准循旬殉潤盾巡遵庶緒叙徐償匠升召奨宵尚床彰抄掌昇晶沼渉焦症硝礁祥称粧紹肖衝訟詔詳鐘丈冗剰壌嬢浄畳譲醸錠嘱飾殖触辱伸侵唇娠寝審慎振浸紳薪診辛震刃尋甚尽迅陣酢吹帥炊睡粋衰遂酔錘随髄崇枢据杉澄瀬畝是姓征牲誓請逝斉隻惜斥析籍跡拙摂窃仙占扇栓潜旋繊薦践遷銑鮮漸禅繕塑措疎礎租粗訴阻僧双喪壮捜掃挿曹槽燥荘葬藻遭霜騒憎贈促即俗賊堕妥惰駄耐怠替泰滞胎袋逮滝卓択拓沢濯託濁諾但奪脱棚丹嘆淡端胆鍛壇弾恥痴稚致遅畜蓄逐秩窒嫡抽衷鋳駐弔彫徴懲挑眺聴脹超跳勅朕沈珍鎮陳津墜塚漬坪釣亭偵貞呈堤帝廷抵締艇訂逓邸泥摘滴哲徹撤迭添殿吐塗斗渡途奴怒倒凍唐塔悼搭桃棟盗痘筒到謄踏逃透陶騰闘洞胴峠匿督篤凸突屯豚曇鈍縄軟尼弐如尿妊忍寧猫粘悩濃把覇婆廃排杯輩培媒賠陪伯拍泊舶薄迫漠爆縛肌鉢髪伐罰抜閥伴帆搬畔繁般藩販範煩頒盤蛮卑妃彼扉披泌疲碑罷被避尾微匹姫漂描苗浜賓頻敏瓶怖扶敷普浮符腐膚譜賦赴附侮舞封伏幅覆払沸噴墳憤紛雰丙併塀幣弊柄壁癖偏遍舗捕穂募慕簿倣俸奉峰崩抱泡砲縫胞芳褒邦飽乏傍剖坊妨帽忙房某冒紡肪膨謀僕墨撲朴没堀奔翻凡盆摩磨魔麻埋膜又抹繭慢漫魅岬妙眠矛霧婿娘銘滅免茂妄猛盲網耗黙戻紋匁厄躍柳愉癒諭唯幽悠憂猶裕誘雄融与誉庸揚揺擁溶窯謡踊抑翼羅裸頼雷絡酪欄濫吏履痢離硫粒隆竜慮虜了僚寮涼猟療糧陵倫厘隣塁涙累励鈴隷零霊麗齢暦劣烈裂廉恋錬炉露廊楼浪漏郎賄惑枠湾腕'),
-        (u'Jinmeiyou', u'阿葵茜渥旭梓絢綾鮎杏伊惟亥郁磯允胤卯丑唄叡瑛艶苑於旺伽嘉茄霞魁凱馨叶樺鎌茅侃莞巌伎嬉毅稀亀誼鞠橘亨匡喬尭桐錦欣欽芹衿玖矩駒熊栗袈圭慧桂拳絃胡虎伍吾梧瑚鯉倖宏弘昂晃浩紘鴻嵯沙瑳裟哉采冴朔笹皐燦獅爾蒔汐鹿偲紗洲峻竣舜駿淳醇曙渚恕庄捷昌梢菖蕉丞穣晋榛秦須翠瑞嵩雛碩曽爽惣綜聡蒼汰黛鯛鷹啄琢只辰巽旦檀智猪暢蝶椎槻蔦椿紬鶴悌汀禎杜藤憧瞳寅酉惇敦奈那凪捺楠虹乃之巴萩肇鳩隼斐緋眉柊彦媛彪彬芙楓蕗碧甫輔朋萌鳳鵬睦槙柾亦麿巳稔椋孟也冶耶弥靖佑宥柚湧祐邑楊耀蓉遥嵐藍蘭李梨璃琉亮凌瞭稜諒遼琳麟瑠伶嶺怜玲蓮呂禄倭亘侑勁奎崚彗昴晏晨晟暉栞椰毬洸洵滉漱澪燎燿瑶皓眸笙綺綸翔脩茉莉菫詢諄赳迪頌颯黎凜熙')
-        ]
