@@ -38,41 +38,38 @@ timeTable = {
     "seconds": lambda n: ngettext("%s second", "%s seconds", n),
     }
 
-shortTimeTable = {
-    "years": "%sy",
-    "months": "%sm",
-    "days": "%sd",
-    "hours": "%sh",
-    "minutes": "%sm",
-    "seconds": "%ss",
+afterTimeTable = {
+    "years": lambda n: ngettext("%s year<!--after>", "%s years<!--after>", n),
+    "months": lambda n: ngettext("%s month<!--after>", "%s months<!--after>", n),
+    "days": lambda n: ngettext("%s day<!--after>", "%s days<!--after>", n),
+    "hours": lambda n: ngettext("%s hour<!--after>", "%s hours<!--after>", n),
+    "minutes": lambda n: ngettext("%s minute<!--after>", "%s minutes<!--after>", n),
+    "seconds": lambda n: ngettext("%s second<!--after>", "%s seconds<!--after>", n),
     }
 
-def fmtTimeSpan(time, pad=0, point=0, short=False):
+shortTimeTable = {
+    "years": _("%sy"),
+    "months": _("%sm"),
+    "days": _("%sd"),
+    "hours": _("%sh"),
+    "minutes": _("%sm"),
+    "seconds": _("%ss"),
+    }
+
+def fmtTimeSpan(time, pad=0, point=0, short=False, after=False):
     "Return a string representing a time span (eg '2 days')."
     (type, point) = optimalPeriod(time, point)
     time = convertSecondsTo(time, type)
     if short:
         fmt = shortTimeTable[type]
     else:
-        fmt = timeTable[type](_pluralCount(round(time, point)))
+        if after:
+            fmt = afterTimeTable[type](_pluralCount(time, point))
+        else:
+            fmt = timeTable[type](_pluralCount(time, point))
     timestr = "%(a)d.%(b)df" % {'a': pad, 'b': point}
     return locale.format_string("%" + (fmt % timestr), time)
 
-def fmtTimeSpanPair(time1, time2, short=False):
-    (type, point) = optimalPeriod(time1, 0)
-    time1 = convertSecondsTo(time1, type)
-    time2 = convertSecondsTo(time2, type)
-    # a pair is always  should always be read as plural
-    if short:
-        fmt = shortTimeTable[type]
-    else:
-        fmt = timeTable[type](2)
-    timestr = "%(a)d.%(b)df" % {'a': pad, 'b': point}
-    finalstr = "%s-%s" % (
-        locale.format_string('%' + timestr, time1),
-        locale.format_string('%' + timestr, time2),)
-    return fmt % finalstr
-
 def optimalPeriod(time, point):
     if abs(time) < 60:
         type = "seconds"
@@ -106,28 +103,29 @@ def convertSecondsTo(seconds, type):
         return seconds / 31536000.0
     assert False
 
-def _pluralCount(time):
-    if round(time, 1) == 1:
-        return 1
-    return round(time, 1)
+def _pluralCount(time, point):
+    if point:
+        return 2
+    return round(time)
 
-# locale
+# Locale
 ##############################################################################
 
 def fmtPercentage(float_value, point=1):
-	"Return string representing a float respecting current locale followed by a percentage sign"
-	fmt = '%' + "0.%(b)df" % {'b': point}
-	return locale.format_string(fmt, float_value) + "%"
+    "Return float with percentage sign"
+    fmt = '%' + "0.%(b)df" % {'b': point}
+    return locale.format_string(fmt, float_value) + "%"
 
 def fmtFloat(float_value, point=1):
-	"Return a string representing a float with decimal separator according to current locale"
-	fmt = '%' + "0.%(b)df" % {'b': point}
-	return locale.format_string(fmt, float_value)
+    "Return a string with decimal separator according to current locale"
+    fmt = '%' + "0.%(b)df" % {'b': point}
+    return locale.format_string(fmt, float_value)
 
 # HTML
 ##############################################################################
 
 def stripHTML(s):
+    s = re.sub("(?s)<style.*?>.*?</style>", "", s)
     s = re.sub("<.*?>", "", s)
     s = s.replace("&lt;", "<")
     s = s.replace("&gt;", ">")
