@@ -28,7 +28,7 @@ def setup1():
 def test_export_anki():
     oldTime = deck.modified
     e = AnkiExporter(deck)
-    newname = unicode(tempfile.mkstemp()[1])
+    newname = unicode(tempfile.mkstemp(prefix="ankitest")[1])
     os.unlink(newname)
     e.exportInto(newname)
     assert deck.modified == oldTime
@@ -36,7 +36,7 @@ def test_export_anki():
     d2 = DeckStorage.Deck(newname)
     assert d2.cardCount == 4
     # try again, limited to a tag
-    newname = unicode(tempfile.mkstemp()[1])
+    newname = unicode(tempfile.mkstemp(prefix="ankitest")[1])
     os.unlink(newname)
     e.limitTags = ['tag']
     e.exportInto(newname)
@@ -46,35 +46,17 @@ def test_export_anki():
 @nose.with_setup(setup1)
 def test_export_textcard():
     e = TextCardExporter(deck)
-    f = unicode(tempfile.mkstemp()[1])
+    f = unicode(tempfile.mkstemp(prefix="ankitest")[1])
     os.unlink(f)
     e.exportInto(f)
     e.includeTags = True
     e.exportInto(f)
 
-#     # test speed
-#     newname = unicode(tempfile.mkstemp()[1])
-#     os.unlink(newname)
-#     d = DeckStorage.Deck("/home/resolve/speed.db")
-#     import time; t = time.time()
-#     e = TextCardExporter(d)
-#     e.exportInto(newname)
-#     print time.time() - t
-
 @nose.with_setup(setup1)
 def test_export_textfact():
     e = TextFactExporter(deck)
-    f = unicode(tempfile.mkstemp()[1])
+    f = unicode(tempfile.mkstemp(prefix="ankitest")[1])
     os.unlink(f)
     e.exportInto(f)
     e.includeTags = True
     e.exportInto(f)
-
-#    # test speed
-#     newname = unicode(tempfile.mkstemp()[1])
-#     os.unlink(newname)
-#     d = DeckStorage.Deck("/home/resolve/speed.db")
-#     import time; t = time.time()
-#     e = TextFactExporter(d)
-#     e.exportInto(newname)
-#     print time.time() - t
